function HW4();
% M-file for modification of problem 5D in Wollenberg and Wood
% Code modifications from Wollenberg m-file by Kevin Tomsovic
% 		- Comments added/modified
% 		- Code debugged (errors in indexing) 
%		- Code made generic (any # of time periods, units) but still no startup cost
%		- Replaced Economic Dispatch with quadratic programming approach. You can use
%           a linearized approach if you prefer
%		- GREATLY cleaned up code
% Other concerns do
% 		- Might be possible to get rid of the for loop in the dispatch

% I/O Setup
echo off
clear session
more on;
format compact

% Problem data (Example 5D in Wollenberg)
a = [500., 300., 100.];			% Constant cost coefficient
b = [10., 8., 6.];				% Linear cost coefficient
c = [0.002, 0.004, 0.006];	% 2nd order cost coefficient
pload = [250, 1050, 350, 150];	% Load
pmin = [100., 50., 100.];		% Min. generator outputs
pmax = [400., 500., 300.];		% Max. generator outputs
T=length(pload);					% Number of time perios
Ng=length(pmax);					% Number of units		

% Algorithm parameters
Maxiter=20;							% Maximum number of iterations
iter=0;								% Iteration count
Tol=0.0001;							% Tolerance on LR iterations
alphaplus=0.01;					% Dual gradient multiplier for increase
alphaneg=0.004;					    % Dual gradient multiplier for decrease
LargeNo=10000;						% Dummy large number for constraint violations

% Initialize dual variables, dual gradient
lambda = zeros(1,T);
graddual=LargeNo*ones(1,T);

% Use the dual gradient and # of iterations as a stopping criteria
while (max(graddual) > Tol) & (iter < Maxiter)		% Stopping criteria
   iter=iter+1;			% Iteration count

   % Perform relaxed unit commitment
   % Commit and dispatch units individually based on dual variables (lambdas)
   % Find incremental cost by dispatching
   % to lambda but respect upper and lower bounds on units
   p=min(ones(T,1)*pmax,max(ones(T,1)*pmin,lambda'*(0.5./c)-ones(T,1)*(b./(2*c))));
   % Turn on units if costs are less than those found from lambda
   % otherwise turn off  
   u= ( (ones(T,1)*a+ones(T,1)*b.*p+ones(T,1)*c.*p.^2)-lambda'*ones(1,Ng).*p < 0);
   
   % Calculate dual for this commitment and dispatch (i.e. the Lagrangian, not truly dual)
   dual = sum(sum(((ones(T,1)*a+ones(T,1)*b.*p+ones(T,1)*c.*p.^2).*u)') ... % Total cost term
      			+  lambda.*(pload - sum((p.*u)')));									 % Constraint term

   % Since dispatch above won't necessarily meet the demand can            
   % Calculate the primal function by dispatching the units for the given commitment
   for(t=1:T)	%	Dispatch each unit at each hour
      	% If capacity not too little and not too much then solve ED
     if (sum(pmin.*u(t,:))<=pload(t) & sum(pmax.*u(t,:))>=pload(t)) 	 
        % Perform economic dispatch using Quadratic Programming 
        p(t,:)=ed(a,b,c,pmin,pmax,pload(t),u(t,:),Ng);
        primal(t)= sum( (a+b.*p(t,:)+c.*p(t,:).^2).*u(t,:) );
     else  % Capacity is too little or too much so set primal to some large number
        primal(t) = LargeNo;
     end 
   end % for t=1:T; All units are dispatched
	primal = sum(primal);

   % Now calculate the gradient of the dual function and update lambda
   % Gradient is simply constraint imbalance
   graddual=pload-sum((p.*u)');
   lambda=lambda+max(alphaplus.*graddual,alphaneg.*graddual)
   u
   
end

% Output results
disp('Results - lambda, commitment');
lambda
u
p=p.*u
iter
disp('Results - duality gap, somewhat misleading due to the dual calculation');
dual
primal
dualgap=(primal-dual)/primal
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% An economic dispatch using Quadratic Programming
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  p=ed(a,b,c,pmin,pmax,pload,u,Ng)
   i=0;
   p=zeros(1,Ng);
	% Set up a quadratic programming problem to find a solution
   for(j=1:Ng)		% Only want the committed units
   	if(u(j) == 1)
      	i=i+1;				% Counting number of units committed
         H(i)=c(j)*2;		% Quadratic costs
         f(i)=b(j);			% Linear costs

         vlb(i)=pmin(j);	% Lower bounds
         vub(i)=pmax(j);	% Upper bound
      end;
   end; % for j=1:Ng
   % Use Matlab quadratic programming algorithm to find a solution
   % pqp=qp(diag(H),f,ones(1,i),pload,vlb,vup,[],1);
   % Suppress display, and warnings about using the large scale method since only 1 equality
   options=optimset('quadprog');
   options=optimset(options,'Display','off','LargeScale','off');
   pqp=quadprog(diag(H),f,[],[],ones(1,i),pload,vlb,vub,[],options);
   i=0;
   for(j=1:Ng)		% Now set the dispatch for all units
  		if(u(j) == 1)
     		i=i+1;				% Counting number of units committed
     	   p(j)=pqp(i);		% Dispatch of unit j
     	end;
  	end; % for j=1:Ng
return
      


